import re
from django.conf import settings
import jwt

SENSITIVE_KEYS = ['password', 'token', 'access', 'refresh']
if hasattr(settings, 'API_LOGGER_EXCLUDE_KEYS'):
    if type(settings.DRF_API_LOGGER_EXCLUDE_KEYS) in (list, tuple):
        SENSITIVE_KEYS.extend(settings.DRF_API_LOGGER_EXCLUDE_KEYS)


def get_headers(request=None):
    """
        Function:       get_headers(self, request)
        Description:    To get all the headers from request
    """
    regex = re.compile('^HTTP_')
    return dict((regex.sub('', header), value) for (header, value)
                in request.META.items() if header.startswith('HTTP_'))


def get_client_ip(request):
    try:
        x_forwarded_for = request.META.get('HTTP_X_FORWARDED_FOR')
        if x_forwarded_for:
            ip = x_forwarded_for.split(',')[0]
        else:
            ip = request.META.get('REMOTE_ADDR')
        return ip
    except:
        return ''


def mask_sensitive_data(data, mask_api_parameters=True):
    """
    Hides sensitive keys specified in sensitive_keys settings.
    Loops recursively over nested dictionaries.
    When the mask_api_parameters parameter is set, the function will
    instead iterate over sensitive_keys and remove them from an api
    URL string.
    """
    if type(data) is not dict:
        if mask_api_parameters and type(data) is str:
            for sensitive_key in SENSITIVE_KEYS:
                data = re.sub('({}=)(.*?)($|&)'.format(sensitive_key),
                              '\g<1>***FILTERED***\g<3>'.format(sensitive_key.upper()), data)
        # new code
        if type(data) is list:
            data = [mask_sensitive_data(item) for item in data]
        return data
    for key, value in data.items():
        if key in SENSITIVE_KEYS:
            data[key] = "***FILTERED***"

        if type(value) is dict:
            data[key] = mask_sensitive_data(data[key])

        if type(value) is list:
            data[key] = [mask_sensitive_data(item) for item in data[key]]

    return data


def decode_jwt_token(request):
    authorization_header = request.headers.get('Authorization', '')
    if authorization_header.startswith('Bearer '):
        token = authorization_header.split(' ')[1]
        try:
            decoded_token = jwt.decode(token, settings.SECRET_KEY, algorithms=['HS256'])
            email = decoded_token.get('email')
            user_id = decoded_token.get('user_id')
            return email, user_id
        except jwt.DecodeError:
            # Handle decoding error (e.g., log it, return default values)
            return '', ''
    else:
        return '', ''



from django.conf import settings
from elasticsearch import Elasticsearch

from core.celery import APP


@APP.task()
def send_logs_to_elk(data):
    try:
        elastic_search = Elasticsearch(hosts=settings.ELK_LOGGER_URL)
        result = elastic_search.index(index=settings.APP_NAME.lower(), body=data)
        return result
    except Exception as e:
        print(f"Failed to index data into Elasticsearch: {e}")



import json
import os

import requests
from drf_spectacular.contrib.rest_framework_simplejwt import SimpleJWTScheme
from rest_framework import serializers
from rest_framework_simplejwt.authentication import JWTAuthentication
from rest_framework_simplejwt.exceptions import AuthenticationFailed


class UserData(object):
    is_authenticated = True

    def __init__(self, my_dict):
        for key in my_dict:
            setattr(self, key, my_dict[key])


def get_auth_user(token):
    auth_service_url = os.getenv('AUTH_SERVICE_BASE_URL', 'http://localhost:10060')
    auth_decode_url = f'{auth_service_url}/api/v1/auth/token/decode/'
    headers = {'Accept': 'application/json', 'Authorization': f'Bearer {str(token)}',
               'Content-Type': 'application/json'}
    data = {'token': str(token)}
    try:
        res = requests.post(url=auth_decode_url, data=json.dumps(data), headers=headers)

    except requests.ConnectionError as err:
        raise serializers.ValidationError(f"Cannot establish connection: {err}") from err

    except requests.HTTPError as err:
        raise serializers.ValidationError(f"HTTP Error: {err}") from err
    except Exception as err:
        raise serializers.ValidationError(f"Error occurred: {err}") from err

    if 200 <= res.status_code < 300:
        return UserData(res.json())
    else:
        raise AuthenticationFailed


class CustomJWTAuthentication(JWTAuthentication):

    def get_validated_token(self, raw_token):
        return raw_token.decode()

    def get_user(self, validated_token):
        return get_auth_user(validated_token)


class CustomJWTAuthenticationScheme(SimpleJWTScheme):
    target_class = CustomJWTAuthentication



import requests


class BaseRequest:
    base_url = None

    def __init__(self, request):
        header = request.META.get("HTTP_AUTHORIZATION", None)
        parts = header.split()
        if len(parts) == 0:
            raise Exception("Empty AUTHORIZATION header sent")
        if len(parts) != 2:
            raise Exception(
                'Authorization header must contain two space-delimited values'
            )
        self.token = parts[1]

    def send_request(self, method, path, data=None):
        headers = {'Accept': 'application/json', 'Authorization': f'Bearer {str(self.token)}',
                   'Content-Type': 'application/json'}
        try:

            res = requests.request(method=method, url=f"{self.base_url}/{path}", json=data, headers=headers, verify=False)
        except Exception as err:
            raise Exception(f"Error occurred: {err}") from err
        if 200 <= res.status_code < 300:
            return res.json()
        else:
            raise Exception(f"Error occurred: {res.text}")
